#define MAX_BLOB_COLUMN_LENGTH  1024

struct ColumnBuffer {
	Tcl_Obj *tclValue;
	SQLPOINTER valueBuffer;
	SQLLEN indicator;
	SQLLEN bufferSize;
	SQLSMALLINT sqlType;
	SQLSMALLINT targetType;
	int boundP;
};

struct ArgDef {
	SQLSMALLINT sqlType;
	SQLUINTEGER columnDef;
	SQLSMALLINT scale;
	SQLSMALLINT nullableP;
	SQLUINTEGER valuePointer;
	SQLSMALLINT sourceType;
};

struct statement {
	SQLHSTMT stmt;
	struct connection *parent;
	enum {STMT_SQL, STMT_TABLES, STMT_COLUMNS, STMT_INDEXES, STMT_TYPEINFO,
	      STMT_PRIMARYKEYS}
		statementType;
	struct ColumnBuffer *columnBuffers;
	SQLSMALLINT numColumns;
	struct ArgDef *argDefs;
	SQLSMALLINT numArgDefs;
	int multiSetsP;
};

static SQLSMALLINT MapSqlType (SQLINTEGER colType);
SQLSMALLINT LookUpSqlType(const char *typeName);
static struct statement *CreateStatement(Tcl_Interp *interp,
				  const struct connection *parent,
				  const char *sql,Tcl_Obj *argtypedefs,
				  char **outmessage);
static void UnbindAndFreeColumnBuffers(struct statement *statement);
static int ExecuteStatement(Tcl_Interp *interp,struct statement *statement,Tcl_Obj *args,char **outmessage);
static int AllocAndBindColumnBuffers(struct statement *statement,char **outmessage);
static int FetchResultRow(struct statement *statement,char **outmessage);
static void DestroyStatement(struct statement *statement);
static int RunStatement(Tcl_Interp *interp,struct statement *statement,
			Tcl_Obj *args);
static int EvalStatement(Tcl_Interp *interp,Tcl_Obj *proc,
			 struct statement *statement,Tcl_Obj *args);
static int ReadStatement(Tcl_Interp *interp,Tcl_Obj *arrayspec,
			 struct statement *statement,Tcl_Obj *args);
static int NumResultCols(struct statement *statement,char **outmessage);
static void EventWait(int delay);

#ifndef _MSC_VER
#define min(a, b) ((a) < (b) ? (a) : (b))
#endif
